'use strict';

var jsx = require('@eslint-react/jsx');
var shared = require('@eslint-react/shared');
var ast = require('@eslint-react/ast');
var core = require('@eslint-react/core');
var effect = require('effect');
var types = require('@typescript-eslint/types');
var _var = require('@eslint-react/var');
var typeUtils = require('@typescript-eslint/type-utils');
var utils = require('@typescript-eslint/utils');
var tsutils = require('ts-api-utils');
var ts = require('typescript');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var tsutils__namespace = /*#__PURE__*/_interopNamespace(tsutils);
var ts__default = /*#__PURE__*/_interopDefault(ts);

// package.json
var name = "eslint-plugin-react-core";
var version = "1.5.17-next.2";
var createRule = shared.createRuleForPlugin("core");

// src/utils/get-children-to-array-selector.ts
function getChildrenToArraySelector() {
  return [
    ":matches(",
    "CallExpression",
    "[callee.object.property.name=Children]",
    "[callee.property.name=toArray]",
    ",",
    "CallExpression",
    "[callee.object.name=Children]",
    "[callee.property.name=toArray]",
    ")"
  ].join("");
}

// src/rules/avoid-shorthand-boolean.ts
var RULE_NAME = "avoid-shorthand-boolean";
var avoid_shorthand_boolean_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce boolean attributes notation in JSX",
      requiresTypeChecking: false
    },
    messages: {
      AVOID_SHORTHAND_BOOLEAN: "Avoid using shorthand boolean attribute '{{propName}}'"
    },
    schema: []
  },
  name: RULE_NAME,
  create(context) {
    return {
      JSXAttribute(node) {
        const { value } = node;
        const propName = jsx.getPropName(node);
        if (value === null) {
          context.report({
            data: {
              propName
            },
            messageId: "AVOID_SHORTHAND_BOOLEAN",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});

// src/rules/avoid-shorthand-fragment.ts
var RULE_NAME2 = "avoid-shorthand-fragment";
var avoid_shorthand_fragment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce using fragment component instead of shorthand fragment syntax",
      requiresTypeChecking: false
    },
    messages: {
      AVOID_SHORTHAND_FRAGMENT: "Avoid shorthand fragment syntax, use fragment component instead."
    },
    schema: []
  },
  name: RULE_NAME2,
  create(context) {
    return {
      JSXFragment(node) {
        context.report({
          messageId: "AVOID_SHORTHAND_FRAGMENT",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME3 = "ensure-forward-ref-using-ref";
var ensure_forward_ref_using_ref_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "require all 'forwardRef' components include a 'ref' parameter",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      ENSURE_FORWARD_REF_USING_REF: "A 'forwardRef' is used with this component but no 'ref' parameter is set."
    },
    schema: []
  },
  name: RULE_NAME3,
  create(context) {
    return {
      CallExpression(node) {
        if (!core.isForwardRefCall(node, context)) return;
        const [component] = node.arguments;
        if (!component || !ast.isFunction(component)) return;
        const [_2, ref] = component.params;
        if (!ref) {
          context.report({
            messageId: "ENSURE_FORWARD_REF_USING_REF",
            node: component
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME4 = "no-access-state-in-setstate";
function isKeyLiteralLike(node, property) {
  return property.type === ast.NodeType.Literal || property.type === ast.NodeType.TemplateLiteral && property.expressions.length === 0 || !node.computed && property.type === ast.NodeType.Identifier;
}
function isThisSetState(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && ast.isThisExpression(callee.object) && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function getName(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return effect.Option.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return effect.Option.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return effect.Option.fromNullable(node.quasis[0]?.value.raw);
  }
  return effect.Option.none();
}
var no_access_state_in_setstate_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow accessing 'this.state' within 'setState'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_ACCESS_STATE_IN_SETSTATE: "Do not access 'this.state' within 'setState'. Use 'setState' callback instead."
    },
    schema: []
  },
  name: RULE_NAME4,
  create(context) {
    const classStack = effect.MutableList.make();
    const methodStack = effect.MutableList.make();
    const setStateStack = effect.MutableList.make();
    return {
      CallExpression(node) {
        if (!isThisSetState(node)) return;
        effect.MutableList.append(setStateStack, [node, false]);
      },
      "CallExpression:exit"(node) {
        if (!isThisSetState(node)) return;
        effect.MutableList.pop(setStateStack);
      },
      ClassDeclaration(node) {
        effect.MutableList.append(classStack, [node, core.isClassComponent(node)]);
      },
      "ClassDeclaration:exit"() {
        effect.MutableList.pop(classStack);
      },
      ClassExpression(node) {
        effect.MutableList.append(classStack, [node, core.isClassComponent(node)]);
      },
      "ClassExpression:exit"() {
        effect.MutableList.pop(classStack);
      },
      MemberExpression(node) {
        if (!ast.isThisExpression(node.object)) return;
        const [currentClass, isComponent] = effect.MutableList.tail(classStack) ?? [];
        if (!currentClass || !isComponent) return;
        const [currentMethod, isStatic] = effect.MutableList.tail(methodStack) ?? [];
        if (!currentMethod || isStatic) return;
        const [setState, hasThisState] = effect.MutableList.tail(setStateStack) ?? [];
        if (!setState || hasThisState) return;
        if (!effect.Option.exists(getName(node.property), (name2) => name2 === "state")) return;
        context.report({ messageId: "NO_ACCESS_STATE_IN_SETSTATE", node });
      },
      MethodDefinition(node) {
        effect.MutableList.append(methodStack, [node, node.static]);
      },
      "MethodDefinition:exit"() {
        effect.MutableList.pop(methodStack);
      },
      PropertyDefinition(node) {
        effect.MutableList.append(methodStack, [node, node.static]);
      },
      "PropertyDefinition:exit"() {
        effect.MutableList.pop(methodStack);
      },
      VariableDeclarator(node) {
        const [currentClass, isComponent] = effect.MutableList.tail(classStack) ?? [];
        if (!currentClass || !isComponent) return;
        const [currentMethod, isStatic] = effect.MutableList.tail(methodStack) ?? [];
        if (!currentMethod || isStatic) return;
        const [setState, hasThisState] = effect.MutableList.tail(setStateStack) ?? [];
        if (!setState || hasThisState) return;
        if (!(node.init && ast.isThisExpression(node.init) && node.id.type === ast.NodeType.ObjectPattern)) return;
        const hasState = node.id.properties.some((prop) => {
          if (prop.type === ast.NodeType.Property && isKeyLiteralLike(prop, prop.key)) {
            return effect.Option.exists(getName(prop.key), (name2) => name2 === "state");
          }
          return false;
        });
        if (!hasState) return;
        context.report({ messageId: "NO_ACCESS_STATE_IN_SETSTATE", node });
      }
    };
  },
  defaultOptions: []
});

// ../../../node_modules/.pnpm/ts-pattern@5.2.0/node_modules/ts-pattern/dist/index.js
var t = Symbol.for("@ts-pattern/matcher");
var e = Symbol.for("@ts-pattern/isVariadic");
var n = "@ts-pattern/anonymous-select-key";
var r = (t2) => Boolean(t2 && "object" == typeof t2);
var i = (e2) => e2 && !!e2[t];
var o = (n2, s2, c2) => {
  if (i(n2)) {
    const e2 = n2[t](), { matched: r2, selections: i2 } = e2.match(s2);
    return r2 && i2 && Object.keys(i2).forEach((t2) => c2(t2, i2[t2])), r2;
  }
  if (r(n2)) {
    if (!r(s2)) return false;
    if (Array.isArray(n2)) {
      if (!Array.isArray(s2)) return false;
      let t2 = [], r2 = [], a2 = [];
      for (const o2 of n2.keys()) {
        const s3 = n2[o2];
        i(s3) && s3[e] ? a2.push(s3) : a2.length ? r2.push(s3) : t2.push(s3);
      }
      if (a2.length) {
        if (a2.length > 1) throw new Error("Pattern error: Using `...P.array(...)` several times in a single pattern is not allowed.");
        if (s2.length < t2.length + r2.length) return false;
        const e2 = s2.slice(0, t2.length), n3 = 0 === r2.length ? [] : s2.slice(-r2.length), i2 = s2.slice(t2.length, 0 === r2.length ? Infinity : -r2.length);
        return t2.every((t3, n4) => o(t3, e2[n4], c2)) && r2.every((t3, e3) => o(t3, n3[e3], c2)) && (0 === a2.length || o(a2[0], i2, c2));
      }
      return n2.length === s2.length && n2.every((t3, e2) => o(t3, s2[e2], c2));
    }
    return Object.keys(n2).every((e2) => {
      const r2 = n2[e2];
      return (e2 in s2 || i(a2 = r2) && "optional" === a2[t]().matcherType) && o(r2, s2[e2], c2);
      var a2;
    });
  }
  return Object.is(s2, n2);
};
var s = (e2) => {
  var n2, o2, a2;
  return r(e2) ? i(e2) ? null != (n2 = null == (o2 = (a2 = e2[t]()).getSelectionKeys) ? void 0 : o2.call(a2)) ? n2 : [] : Array.isArray(e2) ? c(e2, s) : c(Object.values(e2), s) : [];
};
var c = (t2, e2) => t2.reduce((t3, n2) => t3.concat(e2(n2)), []);
function a(...t2) {
  if (1 === t2.length) {
    const [e2] = t2;
    return (t3) => o(e2, t3, () => {
    });
  }
  if (2 === t2.length) {
    const [e2, n2] = t2;
    return o(e2, n2, () => {
    });
  }
  throw new Error(`isMatching wasn't given the right number of arguments: expected 1 or 2, received ${t2.length}.`);
}
function u(t2) {
  return Object.assign(t2, { optional: () => l(t2), and: (e2) => m(t2, e2), or: (e2) => d(t2, e2), select: (e2) => void 0 === e2 ? p(t2) : p(e2, t2) });
}
function h(t2) {
  return Object.assign(((t3) => Object.assign(t3, { [Symbol.iterator]() {
    let n2 = 0;
    const r2 = [{ value: Object.assign(t3, { [e]: true }), done: false }, { done: true, value: void 0 }];
    return { next: () => {
      var t4;
      return null != (t4 = r2[n2++]) ? t4 : r2.at(-1);
    } };
  } }))(t2), { optional: () => h(l(t2)), select: (e2) => h(void 0 === e2 ? p(t2) : p(e2, t2)) });
}
function l(e2) {
  return u({ [t]: () => ({ match: (t2) => {
    let n2 = {};
    const r2 = (t3, e3) => {
      n2[t3] = e3;
    };
    return void 0 === t2 ? (s(e2).forEach((t3) => r2(t3, void 0)), { matched: true, selections: n2 }) : { matched: o(e2, t2, r2), selections: n2 };
  }, getSelectionKeys: () => s(e2), matcherType: "optional" }) });
}
var f = (t2, e2) => {
  for (const n2 of t2) if (!e2(n2)) return false;
  return true;
};
var g = (t2, e2) => {
  for (const [n2, r2] of t2.entries()) if (!e2(r2, n2)) return false;
  return true;
};
function m(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    let n2 = {};
    const r2 = (t3, e3) => {
      n2[t3] = e3;
    };
    return { matched: e2.every((e3) => o(e3, t2, r2)), selections: n2 };
  }, getSelectionKeys: () => c(e2, s), matcherType: "and" }) });
}
function d(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    let n2 = {};
    const r2 = (t3, e3) => {
      n2[t3] = e3;
    };
    return c(e2, s).forEach((t3) => r2(t3, void 0)), { matched: e2.some((e3) => o(e3, t2, r2)), selections: n2 };
  }, getSelectionKeys: () => c(e2, s), matcherType: "or" }) });
}
function y(e2) {
  return { [t]: () => ({ match: (t2) => ({ matched: Boolean(e2(t2)) }) }) };
}
function p(...e2) {
  const r2 = "string" == typeof e2[0] ? e2[0] : void 0, i2 = 2 === e2.length ? e2[1] : "string" == typeof e2[0] ? void 0 : e2[0];
  return u({ [t]: () => ({ match: (t2) => {
    let e3 = { [null != r2 ? r2 : n]: t2 };
    return { matched: void 0 === i2 || o(i2, t2, (t3, n2) => {
      e3[t3] = n2;
    }), selections: e3 };
  }, getSelectionKeys: () => [null != r2 ? r2 : n].concat(void 0 === i2 ? [] : s(i2)) }) });
}
function v(t2) {
  return "number" == typeof t2;
}
function b(t2) {
  return "string" == typeof t2;
}
function w(t2) {
  return "bigint" == typeof t2;
}
var S = u(y(function(t2) {
  return true;
}));
var O2 = S;
var j = (t2) => Object.assign(u(t2), { startsWith: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && t3.startsWith(n2)))));
  var n2;
}, endsWith: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && t3.endsWith(n2)))));
  var n2;
}, minLength: (e2) => j(m(t2, ((t3) => y((e3) => b(e3) && e3.length >= t3))(e2))), length: (e2) => j(m(t2, ((t3) => y((e3) => b(e3) && e3.length === t3))(e2))), maxLength: (e2) => j(m(t2, ((t3) => y((e3) => b(e3) && e3.length <= t3))(e2))), includes: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && t3.includes(n2)))));
  var n2;
}, regex: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && Boolean(t3.match(n2))))));
  var n2;
} });
var E = j(y(b));
var K = (t2) => Object.assign(u(t2), { between: (e2, n2) => K(m(t2, ((t3, e3) => y((n3) => v(n3) && t3 <= n3 && e3 >= n3))(e2, n2))), lt: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 < t3))(e2))), gt: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 > t3))(e2))), lte: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 <= t3))(e2))), gte: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 >= t3))(e2))), int: () => K(m(t2, y((t3) => v(t3) && Number.isInteger(t3)))), finite: () => K(m(t2, y((t3) => v(t3) && Number.isFinite(t3)))), positive: () => K(m(t2, y((t3) => v(t3) && t3 > 0))), negative: () => K(m(t2, y((t3) => v(t3) && t3 < 0))) });
var x = K(y(v));
var A = (t2) => Object.assign(u(t2), { between: (e2, n2) => A(m(t2, ((t3, e3) => y((n3) => w(n3) && t3 <= n3 && e3 >= n3))(e2, n2))), lt: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 < t3))(e2))), gt: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 > t3))(e2))), lte: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 <= t3))(e2))), gte: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 >= t3))(e2))), positive: () => A(m(t2, y((t3) => w(t3) && t3 > 0))), negative: () => A(m(t2, y((t3) => w(t3) && t3 < 0))) });
var P = A(y(w));
var T = u(y(function(t2) {
  return "boolean" == typeof t2;
}));
var k = u(y(function(t2) {
  return "symbol" == typeof t2;
}));
var B = u(y(function(t2) {
  return null == t2;
}));
var _ = u(y(function(t2) {
  return null != t2;
}));
var N = { __proto__: null, matcher: t, optional: l, array: function(...e2) {
  return h({ [t]: () => ({ match: (t2) => {
    if (!Array.isArray(t2)) return { matched: false };
    if (0 === e2.length) return { matched: true };
    const n2 = e2[0];
    let r2 = {};
    if (0 === t2.length) return s(n2).forEach((t3) => {
      r2[t3] = [];
    }), { matched: true, selections: r2 };
    const i2 = (t3, e3) => {
      r2[t3] = (r2[t3] || []).concat([e3]);
    };
    return { matched: t2.every((t3) => o(n2, t3, i2)), selections: r2 };
  }, getSelectionKeys: () => 0 === e2.length ? [] : s(e2[0]) }) });
}, set: function(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    if (!(t2 instanceof Set)) return { matched: false };
    let n2 = {};
    if (0 === t2.size) return { matched: true, selections: n2 };
    if (0 === e2.length) return { matched: true };
    const r2 = (t3, e3) => {
      n2[t3] = (n2[t3] || []).concat([e3]);
    }, i2 = e2[0];
    return { matched: f(t2, (t3) => o(i2, t3, r2)), selections: n2 };
  }, getSelectionKeys: () => 0 === e2.length ? [] : s(e2[0]) }) });
}, map: function(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    if (!(t2 instanceof Map)) return { matched: false };
    let n2 = {};
    if (0 === t2.size) return { matched: true, selections: n2 };
    const r2 = (t3, e3) => {
      n2[t3] = (n2[t3] || []).concat([e3]);
    };
    if (0 === e2.length) return { matched: true };
    var i2;
    if (1 === e2.length) throw new Error(`\`P.map\` wasn't given enough arguments. Expected (key, value), received ${null == (i2 = e2[0]) ? void 0 : i2.toString()}`);
    const [s2, c2] = e2;
    return { matched: g(t2, (t3, e3) => {
      const n3 = o(s2, e3, r2), i3 = o(c2, t3, r2);
      return n3 && i3;
    }), selections: n2 };
  }, getSelectionKeys: () => 0 === e2.length ? [] : [...s(e2[0]), ...s(e2[1])] }) });
}, intersection: m, union: d, not: function(e2) {
  return u({ [t]: () => ({ match: (t2) => ({ matched: !o(e2, t2, () => {
  }) }), getSelectionKeys: () => [], matcherType: "not" }) });
}, when: y, select: p, any: S, _: O2, string: E, number: x, bigint: P, boolean: T, symbol: k, nullish: B, nonNullable: _, instanceOf: function(t2) {
  return u(y(/* @__PURE__ */ function(t3) {
    return (e2) => e2 instanceof t3;
  }(t2)));
}, shape: function(t2) {
  return u(y(a(t2)));
} };
var W = { matched: false, value: void 0 };
function $(t2) {
  return new z(t2, W);
}
var z = class _z {
  constructor(t2, e2) {
    this.input = void 0, this.state = void 0, this.input = t2, this.state = e2;
  }
  with(...t2) {
    if (this.state.matched) return this;
    const e2 = t2[t2.length - 1], r2 = [t2[0]];
    let i2;
    3 === t2.length && "function" == typeof t2[1] ? i2 = t2[1] : t2.length > 2 && r2.push(...t2.slice(1, t2.length - 1));
    let s2 = false, c2 = {};
    const a2 = (t3, e3) => {
      s2 = true, c2[t3] = e3;
    }, u2 = !r2.some((t3) => o(t3, this.input, a2)) || i2 && !Boolean(i2(this.input)) ? W : { matched: true, value: e2(s2 ? n in c2 ? c2[n] : c2 : this.input, this.input) };
    return new _z(this.input, u2);
  }
  when(t2, e2) {
    if (this.state.matched) return this;
    const n2 = Boolean(t2(this.input));
    return new _z(this.input, n2 ? { matched: true, value: e2(this.input, this.input) } : W);
  }
  otherwise(t2) {
    return this.state.matched ? this.state.value : t2(this.input);
  }
  exhaustive() {
    if (this.state.matched) return this.state.value;
    let t2;
    try {
      t2 = JSON.stringify(this.input);
    } catch (e2) {
      t2 = this.input;
    }
    throw new Error(`Pattern matching error: no pattern matches value ${t2}`);
  }
  run() {
    return this.exhaustive();
  }
  returnType() {
    return this;
  }
};

// src/rules/no-array-index-key.ts
var RULE_NAME5 = "no-array-index-key";
var reactChildrenMethod = ["forEach", "map"];
function isReactChildrenMethod(name2) {
  return reactChildrenMethod.some((method) => method === name2);
}
var iteratorFunctionIndexParamPosition = {
  every: 1,
  filter: 1,
  find: 1,
  findIndex: 1,
  findLast: 1,
  findLastIndex: 1,
  flatMap: 1,
  forEach: 1,
  map: 1,
  reduce: 2,
  reduceRight: 2,
  some: 1
};
function isUsingReactChildren(node, context) {
  const { callee } = node;
  if (!("property" in callee) || !("object" in callee) || !("name" in callee.property)) {
    return false;
  }
  if (!isReactChildrenMethod(callee.property.name)) return false;
  const initialScope = context.sourceCode.getScope(node);
  return $(callee.object).with({ type: ast.NodeType.Identifier, name: "Children" }, () => true).with(
    { type: ast.NodeType.MemberExpression, object: { type: ast.NodeType.Identifier, name: N.string } },
    ({ object }) => core.isInitializedFromReact(object.name, context, initialScope)
  ).otherwise(() => false);
}
function getMapIndexParamName(node, context) {
  const { callee } = node;
  if (callee.type !== ast.NodeType.MemberExpression) return effect.Option.none();
  if (callee.property.type !== ast.NodeType.Identifier) return effect.Option.none();
  const { name: name2 } = callee.property;
  if (!effect.Record.has(iteratorFunctionIndexParamPosition, name2)) return effect.Option.none();
  const callbackArg = node.arguments[isUsingReactChildren(node, context) ? 1 : 0];
  if (!callbackArg) return effect.Option.none();
  if (!ast.isOneOf([ast.NodeType.ArrowFunctionExpression, ast.NodeType.FunctionExpression])(callbackArg)) return effect.Option.none();
  const { params } = callbackArg;
  const maybeIndexParamPosition = effect.Record.get(iteratorFunctionIndexParamPosition, name2);
  if (effect.Option.isNone(maybeIndexParamPosition)) return effect.Option.none();
  const indexParamPosition = maybeIndexParamPosition.value;
  if (params.length < indexParamPosition + 1) return effect.Option.none();
  const param = params.at(indexParamPosition);
  return param && "name" in param ? effect.Option.some(param.name) : effect.Option.none();
}
function getIdentifiersFromBinaryExpression(side) {
  if (side.type === ast.NodeType.Identifier) return [side];
  if (side.type === ast.NodeType.BinaryExpression) {
    return [
      ...getIdentifiersFromBinaryExpression(side.left),
      ...getIdentifiersFromBinaryExpression(side.right)
    ].filter(Boolean);
  }
  return [];
}
var no_array_index_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow using Array index as key",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_ARRAY_INDEX_KEY: "Do not use Array index as key."
    },
    schema: []
  },
  name: RULE_NAME5,
  create(context) {
    const indexParamNames = [];
    function pushIndexParamName(node) {
      effect.Option.map(getMapIndexParamName(node, context), (name2) => indexParamNames.push(name2));
    }
    function popIndexParamName(node) {
      effect.Option.map(getMapIndexParamName(node, context), () => indexParamNames.pop());
    }
    function isArrayIndex(node) {
      return node.type === ast.NodeType.Identifier && indexParamNames.some((name2) => name2 === node.name);
    }
    function checkPropValue(node) {
      if (isArrayIndex(node)) {
        return [{ messageId: "NO_ARRAY_INDEX_KEY", node }];
      }
      if (ast.isOneOf([ast.NodeType.TemplateLiteral, ast.NodeType.BinaryExpression])(node)) {
        const exps = ast.NodeType.TemplateLiteral === node.type ? node.expressions : getIdentifiersFromBinaryExpression(node);
        return exps.reduce((acc, exp) => {
          if (isArrayIndex(exp)) return [...acc, { messageId: "NO_ARRAY_INDEX_KEY", node: exp }];
          return acc;
        }, []);
      }
      const isToStringCall = a({
        type: ast.NodeType.CallExpression,
        callee: {
          type: ast.NodeType.MemberExpression,
          property: {
            type: ast.NodeType.Identifier,
            name: "toString"
          }
        }
      }, node);
      if (isToStringCall) {
        if (!("object" in node.callee && isArrayIndex(node.callee.object))) return [];
        return [{ messageId: "NO_ARRAY_INDEX_KEY", node: node.callee.object }];
      }
      const isStringCall = a({
        type: ast.NodeType.CallExpression,
        callee: {
          type: ast.NodeType.Identifier,
          name: "String"
        }
      }, node);
      if (isStringCall) {
        const [arg] = node.arguments;
        if (arg && isArrayIndex(arg)) return [{ messageId: "NO_ARRAY_INDEX_KEY", node: arg }];
      }
      return [];
    }
    return {
      CallExpression(node) {
        if ((core.isCreateElementCall(node, context) || core.isCloneElementCall(node, context)) && node.arguments.length > 1) {
          if (indexParamNames.length === 0) return;
          const props = node.arguments[1];
          if (props?.type !== ast.NodeType.ObjectExpression) return;
          for (const prop of props.properties) {
            if (!a({ key: { name: "key" } }, prop)) continue;
            if (!("value" in prop)) continue;
            const descriptors = checkPropValue(prop.value);
            for (const descriptor of descriptors) {
              context.report(descriptor);
            }
          }
        }
        pushIndexParamName(node);
      },
      "CallExpression:exit": popIndexParamName,
      JSXAttribute(node) {
        if (node.name.name !== "key") return;
        if (indexParamNames.length === 0) return;
        const { value } = node;
        if (value?.type !== ast.NodeType.JSXExpressionContainer) return;
        const descriptors = checkPropValue(value.expression);
        for (const descriptor of descriptors) {
          context.report(descriptor);
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME6 = "no-children-count";
var no_children_count_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.count'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_COUNT: "Using 'Children.count' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME6,
  create(context) {
    return {
      MemberExpression(node) {
        if (core.isChildrenCount(node, context)) {
          context.report({
            messageId: "NO_CHILDREN_COUNT",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME7 = "no-children-for-each";
var no_children_for_each_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.forEach'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_FOR_EACH: "Using 'Children.forEach' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME7,
  create(context) {
    return {
      MemberExpression(node) {
        if (core.isChildrenForEach(node, context)) {
          context.report({
            messageId: "NO_CHILDREN_FOR_EACH",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME8 = "no-children-map";
var no_children_map_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.map'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_MAP: "Using 'Children.map' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME8,
  create(context) {
    return {
      MemberExpression(node) {
        if (core.isChildrenMap(node, context)) {
          context.report({
            messageId: "NO_CHILDREN_MAP",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME9 = "no-children-only";
var no_children_only_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.only'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_ONLY: "Using 'Children.only' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME9,
  create(context) {
    return {
      MemberExpression(node) {
        if (core.isChildrenOnly(node, context)) {
          context.report({
            messageId: "NO_CHILDREN_ONLY",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME10 = "no-children-prop";
var no_children_prop_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow passing of 'children' as props",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_PROP: "Do not pass children as props."
    },
    schema: []
  },
  name: RULE_NAME10,
  create(context) {
    return {
      CallExpression(node) {
        if (node.arguments.length === 0) return;
        const initialScope = context.sourceCode.getScope(node);
        if (!core.isCreateElementCall(node, context)) return;
        const [_2, props] = node.arguments;
        if (!props || props.type !== ast.NodeType.ObjectExpression) return;
        effect.Option.map(jsx.findPropInProperties(props.properties, context, initialScope)("children"), (prop) => {
          context.report({
            messageId: "NO_CHILDREN_PROP",
            node: prop
          });
        });
      },
      JSXElement(node) {
        const initialScope = context.sourceCode.getScope(node);
        effect.Option.map(jsx.getProp(node.openingElement.attributes, "children", context, initialScope), (prop) => {
          context.report({
            messageId: "NO_CHILDREN_PROP",
            node: prop
          });
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME11 = "no-children-to-array";
var no_children_to_array_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.toArray'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_TO_ARRAY: "Using 'Children.toArray' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME11,
  create(context) {
    return {
      MemberExpression(node) {
        if (core.isChildrenToArray(node, context)) {
          context.report({
            messageId: "NO_CHILDREN_TO_ARRAY",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME12 = "no-class-component";
var isComponentDidCatch = a({
  key: {
    type: ast.NodeType.Identifier,
    name: "componentDidCatch"
  },
  type: N.union(ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition),
  static: false
});
var isGetDerivedStateFromError = a({
  key: {
    type: ast.NodeType.Identifier,
    name: "getDerivedStateFromError"
  },
  type: N.union(ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition),
  static: true
});
var no_class_component_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow class component"
    },
    messages: {
      NO_CLASS_COMPONENT: "Do not use class components. Use function components instead."
    },
    schema: []
  },
  name: RULE_NAME12,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { name: name2, node: component } of components.values()) {
          if (component.body.body.some((m2) => isComponentDidCatch(m2) || isGetDerivedStateFromError(m2))) continue;
          context.report({
            data: {
              name: effect.Option.getOrElse(effect.Function.constant("anonymous"))(name2)
            },
            messageId: "NO_CLASS_COMPONENT",
            node: component
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME13 = "no-clone-element";
var no_clone_element_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'cloneElement'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CLONE_ELEMENT: "Using 'cloneElement' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME13,
  create(context) {
    return {
      CallExpression(node) {
        if (!core.isCloneElementCall(node, context)) return;
        context.report({
          messageId: "NO_CLONE_ELEMENT",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME14 = "no-comment-textnodes";
var no_comment_textnodes_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow comments from being inserted as text nodes",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMMENT_TEXTNODES: "Possible misused comment in text node. Comments inside children section of tag should be placed inside braces."
    },
    schema: []
  },
  name: RULE_NAME14,
  create(context) {
    function checkText(node) {
      if (ast.isOneOf([types.AST_NODE_TYPES.JSXAttribute, types.AST_NODE_TYPES.JSXExpressionContainer])(node.parent)) return false;
      const rawValue = context.sourceCode.getText(node);
      return /^\s*\/(\/|\*)/mu.test(rawValue) && node.parent.type.includes("JSX");
    }
    const check = (node) => {
      if (!ast.isOneOf([types.AST_NODE_TYPES.JSXElement, types.AST_NODE_TYPES.JSXFragment])(node.parent)) return;
      if (!checkText(node)) return;
      context.report({
        messageId: "NO_COMMENT_TEXTNODES",
        node
      });
    };
    return {
      JSXText: check,
      Literal: check
    };
  },
  defaultOptions: []
});
var RULE_NAME15 = "no-complicated-conditional-rendering";
var no_complicated_conditional_rendering_default = createRule({
  meta: {
    type: "problem",
    deprecated: true,
    docs: {
      description: "disallow complicated conditional rendering",
      requiresTypeChecking: true
    },
    messages: {
      NO_COMPLICATED_CONDITIONAL_RENDERING: "Avoid complicated conditional rendering. Extract the logic into separate elements or components."
    },
    schema: []
  },
  name: RULE_NAME15,
  create(context) {
    function check(node) {
      const jsxExpContainer = node.parent?.parent;
      if (!ast.is(ast.NodeType.JSXExpressionContainer)(jsxExpContainer)) return;
      if (!ast.isOneOf([ast.NodeType.JSXElement, ast.NodeType.JSXFragment])(jsxExpContainer.parent)) return;
      if (!jsxExpContainer.parent.children.includes(jsxExpContainer)) return;
      context.report({ messageId: "NO_COMPLICATED_CONDITIONAL_RENDERING", node: jsxExpContainer });
    }
    return {
      "JSXExpressionContainer > ConditionalExpression > ConditionalExpression": check,
      "JSXExpressionContainer > ConditionalExpression > LogicalExpression": check,
      "JSXExpressionContainer > LogicalExpression > ConditionalExpression": check,
      "JSXExpressionContainer > LogicalExpression[operator='&&'] > LogicalExpression[operator='||']": check,
      "JSXExpressionContainer > LogicalExpression[operator='||'] > LogicalExpression[operator='&&']": check
    };
  },
  defaultOptions: []
});
var RULE_NAME16 = "no-component-will-mount";
function isComponentWillMount(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillMount";
}
var no_component_will_mount_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'componentWillMount'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMPONENT_WILL_MOUNT: "[Deprecated] Use 'UNSAFE_componentWillMount' instead."
    },
    schema: []
  },
  name: RULE_NAME16,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isComponentWillMount(member)) {
              context.report({
                messageId: "NO_COMPONENT_WILL_MOUNT",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME17 = "no-component-will-receive-props";
function isComponentWillUpdate(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillReceiveProps";
}
var no_component_will_receive_props_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'componentWillReceiveProps'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMPONENT_WILL_RECEIVE_PROPS: "[Deprecated] Use 'UNSAFE_componentWillReceiveProps' instead."
    },
    schema: []
  },
  name: RULE_NAME17,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isComponentWillUpdate(member)) {
              context.report({
                messageId: "NO_COMPONENT_WILL_RECEIVE_PROPS",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME18 = "no-component-will-update";
function isComponentWillUpdate2(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillUpdate";
}
var no_component_will_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'componentWillUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMPONENT_WILL_UPDATE: "[Deprecated] Use 'UNSAFE_componentWillUpdate' instead."
    },
    schema: []
  },
  name: RULE_NAME18,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isComponentWillUpdate2(member)) {
              context.report({
                messageId: "NO_COMPONENT_WILL_UPDATE",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME19 = "no-create-ref";
var no_create_ref_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'createRef' in function components",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CREATE_REF: "A 'createRef' is not allowed in function components. Use 'useRef' instead."
    },
    schema: []
  },
  name: RULE_NAME19,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const possibleCreateRefCalls = /* @__PURE__ */ new Map();
    return {
      ...listeners,
      CallExpression(node) {
        if (!core.isCreateRefCall(node, context)) return;
        effect.Option.map(
          ctx.getCurrentFunction(),
          ([currentFn]) => possibleCreateRefCalls.set(currentFn, [...possibleCreateRefCalls.get(currentFn) ?? [], node])
        );
      },
      "Program:exit"(node) {
        const components = Array.from(ctx.getAllComponents(node).values());
        for (const { node: component } of components) {
          const createRefCalls = possibleCreateRefCalls.get(component);
          if (!createRefCalls) continue;
          for (const createRefCall of createRefCalls) {
            context.report({
              messageId: "NO_CREATE_REF",
              node: createRefCall
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME20 = "no-direct-mutation-state";
function getName2(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName2(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return effect.Option.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return effect.Option.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return effect.Option.fromNullable(node.quasis[0]?.value.raw);
  }
  return effect.Option.none();
}
function isAssignmentToThisState(node) {
  const { left } = node;
  return left.type === ast.NodeType.MemberExpression && ast.isThisExpression(left.object) && effect.Option.exists(getName2(left.property), (name2) => name2 === "state");
}
function isConstructorFunction(node) {
  return ast.isOneOf([ast.NodeType.FunctionDeclaration, ast.NodeType.FunctionExpression])(node) && ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node.parent) && node.parent.key.type === ast.NodeType.Identifier && node.parent.key.name === "constructor" && node.parent.parent.type === ast.NodeType.ClassBody;
}
var no_direct_mutation_state_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow direct mutation of state",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_DIRECT_MUTATION_STATE: "Do not mutate state directly. Use 'setState()' instead."
    },
    schema: []
  },
  name: RULE_NAME20,
  create(context) {
    return {
      AssignmentExpression(node) {
        if (!isAssignmentToThisState(node)) return;
        const maybeParentClass = ast.traverseUpGuard(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (effect.Option.isNone(maybeParentClass)) return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass)) return;
        const maybeParentConstructor = ast.traverseUpGuard(node, isConstructorFunction);
        if (effect.Option.exists(maybeParentConstructor, (n2) => context.sourceCode.getScope(node).block === n2)) return;
        context.report({
          messageId: "NO_DIRECT_MUTATION_STATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME21 = "no-duplicate-key";
var no_duplicate_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow duplicate keys in 'key' prop when rendering list",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_DUPLICATE_KEY: "A key must be unique. '{{value}}' is duplicated."
    },
    schema: []
  },
  name: RULE_NAME21,
  create(context) {
    const childrenToArraySelector = getChildrenToArraySelector();
    const isWithinChildrenToArrayRef = effect.MutableRef.make(false);
    function checkIteratorElement(node) {
      if (node.type !== ast.NodeType.JSXElement) return effect.Option.none();
      const initialScope = context.sourceCode.getScope(node);
      return effect.Function.pipe(
        jsx.findPropInAttributes(node.openingElement.attributes, context, initialScope)("key"),
        effect.Option.flatMap((k2) => "value" in k2 ? effect.Option.fromNullable(k2.value) : effect.Option.none()),
        effect.Option.flatMap((v2) => {
          return ast.isNodeEqual(v2, v2) ? effect.Option.some({
            data: {
              value: context.sourceCode.getText(v2)
            },
            messageId: "NO_DUPLICATE_KEY",
            node: v2
          }) : effect.Option.none();
        })
      );
    }
    function checkExpression(node) {
      return $(node).with({ type: ast.NodeType.JSXElement }, checkIteratorElement).with({ type: ast.NodeType.JSXFragment }, checkIteratorElement).with({ type: ast.NodeType.ConditionalExpression }, (n2) => {
        if (!("consequent" in n2)) return effect.Option.none();
        return effect.Option.orElse(checkIteratorElement(n2.consequent), () => checkIteratorElement(n2.alternate));
      }).with({ type: ast.NodeType.LogicalExpression }, (n2) => {
        if (!("left" in n2)) return effect.Option.none();
        return effect.Option.orElse(checkIteratorElement(n2.left), () => checkIteratorElement(n2.right));
      }).otherwise(effect.Option.none);
    }
    function checkBlockStatement(node) {
      return ast.getNestedReturnStatements(node).reduce((acc, statement) => {
        if (!statement.argument) return acc;
        const maybeDescriptor = checkIteratorElement(statement.argument);
        if (effect.Option.isNone(maybeDescriptor)) return acc;
        const descriptor = maybeDescriptor.value;
        return [...acc, descriptor];
      }, []);
    }
    const seen = /* @__PURE__ */ new WeakSet();
    return {
      [`${childrenToArraySelector}:exit`]() {
        effect.MutableRef.set(isWithinChildrenToArrayRef, false);
      },
      "ArrayExpression, JSXElement > JSXElement"(node) {
        if (effect.MutableRef.get(isWithinChildrenToArrayRef)) return;
        const elements = $(node).with({ type: ast.NodeType.ArrayExpression }, ({ elements: elements2 }) => elements2).with({ type: ast.NodeType.JSXElement }, ({ parent }) => "children" in parent ? parent.children : []).otherwise(() => []).filter(ast.is(ast.NodeType.JSXElement)).filter((element) => !seen.has(element));
        const keys = elements.reduce(
          (acc, element) => {
            const attr = element.openingElement.attributes.findLast(a({
              type: ast.NodeType.JSXAttribute,
              name: {
                name: "key"
              }
            }));
            if (!attr || !("value" in attr) || attr.value === null) return acc;
            const { value } = attr;
            if (acc.length === 0) return [[element, attr, value]];
            if (acc.some(([_2, _1, v2]) => ast.isNodeEqual(v2, value))) {
              return [...acc, [element, attr, value]];
            }
            return acc;
          },
          []
        );
        if (keys.length < 2) return;
        for (const [element, attr, value] of keys) {
          seen.add(element);
          context.report({
            data: {
              value: context.sourceCode.getText(value)
            },
            messageId: "NO_DUPLICATE_KEY",
            node: attr
          });
        }
      },
      CallExpression(node) {
        const isMapCall = a({
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "map"
            }
          }
        }, node);
        const isArrayFromCall = a({
          type: ast.NodeType.CallExpression,
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "from"
            }
          }
        }, node);
        if (!isMapCall && !isArrayFromCall) return;
        if (effect.MutableRef.get(isWithinChildrenToArrayRef)) return;
        const fn = node.arguments[isMapCall ? 0 : 1];
        if (!ast.isOneOf([ast.NodeType.ArrowFunctionExpression, ast.NodeType.FunctionExpression])(fn)) return;
        if (fn.body.type === ast.NodeType.BlockStatement) {
          for (const descriptor of checkBlockStatement(fn.body)) {
            context.report(descriptor);
          }
          return;
        }
        effect.Option.map(checkExpression(fn.body), context.report);
      },
      [childrenToArraySelector]() {
        effect.MutableRef.set(isWithinChildrenToArrayRef, true);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME22 = "no-implicit-key";
var no_implicit_key_default = createRule({
  meta: {
    type: "problem",
    deprecated: true,
    docs: {
      description: "disallow spreading 'key' from objects.",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_IMPLICIT_KEY: "Prefer specifying key explicitly instead of spreading it from object."
    },
    schema: []
  },
  name: RULE_NAME22,
  create(context) {
    function check(node) {
      const initialScope = context.sourceCode.getScope(node);
      return effect.Function.pipe(
        jsx.findPropInAttributes(node.attributes, context, initialScope)("key"),
        effect.Option.filter(ast.is(ast.NodeType.JSXSpreadAttribute)),
        effect.Option.map((key) => ({
          messageId: "NO_IMPLICIT_KEY",
          node: key
        }))
      );
    }
    return {
      JSXOpeningElement: effect.Function.flow(check, effect.Option.map(context.report))
    };
  },
  defaultOptions: []
});
var RULE_NAME23 = "no-leaked-conditional-rendering";
var allowedVariants = [
  "nullish",
  "boolean",
  "string",
  "object",
  "any",
  "falsy string",
  "truthy boolean",
  "truthy string"
];
function inspectVariantTypes(types) {
  const variantTypes = /* @__PURE__ */ new Set();
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.Unknown))) {
    variantTypes.add("unknown");
    return variantTypes;
  }
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.Null | ts__default.default.TypeFlags.Undefined | ts__default.default.TypeFlags.VoidLike))) {
    variantTypes.add("nullish");
  }
  const booleans = types.filter((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.BooleanLike));
  switch (true) {
    case (booleans.length === 1 && !!booleans[0]): {
      const [first] = booleans;
      effect.Function.pipe(
        $(first).when(tsutils__namespace.isTrueLiteralType, () => effect.Option.some("truthy boolean")).when(tsutils__namespace.isFalseLiteralType, () => effect.Option.some("falsy boolean")).otherwise(effect.Option.none),
        effect.Option.map((v2) => variantTypes.add(v2))
      );
      break;
    }
    case booleans.length === 2: {
      variantTypes.add("boolean");
      break;
    }
  }
  const strings = types.filter((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.StringLike));
  if (strings.length > 0) {
    const evaluated = $(strings).when(
      (types2) => types2.every((type) => type.isStringLiteral() && type.value !== ""),
      effect.Function.constant("truthy string")
    ).when(
      (types2) => types2.every((type) => type.isStringLiteral() && type.value === ""),
      effect.Function.constant("falsy string")
    ).otherwise(effect.Function.constant("string"));
    variantTypes.add(evaluated);
  }
  const numbers = types.filter(
    (type) => tsutils__namespace.isTypeFlagSet(
      type,
      ts__default.default.TypeFlags.NumberLike | ts__default.default.TypeFlags.BigIntLike
    )
  );
  if (numbers.length > 0) {
    const evaluated = $(numbers).when(
      (types2) => types2.every((type) => type.isNumberLiteral() && type.value !== 0),
      effect.Function.constant("truthy number")
    ).when(
      (types2) => types2.every((type) => type.isNumberLiteral() && type.value === 0),
      effect.Function.constant("falsy number")
    ).otherwise(effect.Function.constant("number"));
    variantTypes.add(evaluated);
  }
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.EnumLike))) {
    variantTypes.add("enum");
  }
  if (types.some(
    (type) => !tsutils__namespace.isTypeFlagSet(
      type,
      ts__default.default.TypeFlags.Null | ts__default.default.TypeFlags.Undefined | ts__default.default.TypeFlags.VoidLike | ts__default.default.TypeFlags.BooleanLike | ts__default.default.TypeFlags.StringLike | ts__default.default.TypeFlags.NumberLike | ts__default.default.TypeFlags.BigIntLike | ts__default.default.TypeFlags.TypeParameter | ts__default.default.TypeFlags.Any | ts__default.default.TypeFlags.Unknown | ts__default.default.TypeFlags.Never
    )
  )) {
    variantTypes.add("object");
  }
  if (types.some(
    (type) => tsutils__namespace.isTypeFlagSet(
      type,
      ts__default.default.TypeFlags.TypeParameter | ts__default.default.TypeFlags.Any | ts__default.default.TypeFlags.Unknown
    )
  )) {
    variantTypes.add("any");
  }
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.Never))) {
    variantTypes.add("never");
  }
  return variantTypes;
}
var no_leaked_conditional_rendering_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow problematic leaked values from being rendered",
      recommended: "recommended",
      requiresTypeChecking: true
    },
    messages: {
      NO_LEAKED_CONDITIONAL_RENDERING: "Potential leaked value that might cause unintentionally rendered values or rendering crashes"
    },
    schema: []
  },
  name: RULE_NAME23,
  create(context) {
    const services = utils.ESLintUtils.getParserServices(context);
    function checkExpression(node) {
      return $(node).when(ast.isJSX, effect.Option.none).with({ type: ast.NodeType.LogicalExpression, operator: "&&" }, ({ left, right }) => {
        const isLeftUnaryNot = a({ type: ast.NodeType.UnaryExpression, operator: "!" }, left);
        if (isLeftUnaryNot) return checkExpression(right);
        const leftType = typeUtils.getConstrainedTypeAtLocation(services, left);
        const leftTypeVariants = inspectVariantTypes(tsutils__namespace.unionTypeParts(leftType));
        const isLeftValid = Array.from(leftTypeVariants.values()).every((type) => allowedVariants.some((allowed) => allowed === type));
        if (isLeftValid) return checkExpression(right);
        return effect.Option.some({
          messageId: "NO_LEAKED_CONDITIONAL_RENDERING",
          node: left
        });
      }).with({ type: ast.NodeType.ConditionalExpression }, ({ alternate, consequent }) => {
        return effect.Option.orElse(checkExpression(consequent), () => checkExpression(alternate));
      }).with({ type: ast.NodeType.Identifier }, (n2) => {
        const initialScope = context.sourceCode.getScope(n2);
        return effect.Function.pipe(
          _var.findVariable(n2.name, initialScope),
          effect.Option.flatMap(_var.getVariableInitExpression(0)),
          effect.Option.flatMap(checkExpression)
        );
      }).otherwise(effect.Option.none);
    }
    return {
      "JSXExpressionContainer > ConditionalExpression"(node) {
        effect.Option.map(checkExpression(node), context.report);
      },
      "JSXExpressionContainer > LogicalExpression"(node) {
        effect.Option.map(checkExpression(node), context.report);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME24 = "no-missing-component-display-name";
var no_missing_component_display_name_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "require 'displayName' for memo and forwardRef components",
      requiresTypeChecking: false
    },
    messages: {
      NO_MISSING_COMPONENT_DISPLAY_NAME: "Add missing 'displayName' for component."
    },
    schema: []
  },
  name: RULE_NAME24,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { displayName, flag, node: node2 } of components.values()) {
          const isMemoOrForwardRef = Boolean(flag & core.ERFunctionComponentFlag.ForwardRef) || Boolean(flag & core.ERFunctionComponentFlag.Memo);
          if (effect.Option.isSome(ast.getFunctionIdentifier(node2))) continue;
          if (!isMemoOrForwardRef) continue;
          if (effect.Option.isNone(displayName)) {
            context.report({
              messageId: "NO_MISSING_COMPONENT_DISPLAY_NAME",
              node: node2
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME25 = "no-missing-key";
var no_missing_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "require 'key' prop when rendering list",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_MISSING_KEY: "Missing 'key' prop for element when rendering list",
      NO_MISSING_KEY_WITH_FRAGMENT: "Use fragment component instead of '<>' because it does not support key prop"
    },
    schema: []
  },
  name: RULE_NAME25,
  create(context) {
    const childrenToArraySelector = getChildrenToArraySelector();
    const isWithinChildrenToArrayRef = effect.MutableRef.make(false);
    function checkIteratorElement(node) {
      const initialScope = context.sourceCode.getScope(node);
      if (node.type === ast.NodeType.JSXElement && !jsx.hasProp(node.openingElement.attributes, "key", context, initialScope)) {
        return effect.Option.some({
          messageId: "NO_MISSING_KEY",
          node
        });
      }
      if (node.type === ast.NodeType.JSXFragment) {
        return effect.Option.some({
          messageId: "NO_MISSING_KEY_WITH_FRAGMENT",
          node
        });
      }
      return effect.Option.none();
    }
    function checkExpression(node) {
      return $(node).with({ type: ast.NodeType.JSXElement }, checkIteratorElement).with({ type: ast.NodeType.JSXFragment }, checkIteratorElement).with({ type: ast.NodeType.ConditionalExpression }, (n2) => {
        if (!("consequent" in n2)) return effect.Option.none();
        return effect.Option.orElse(checkIteratorElement(n2.consequent), () => checkIteratorElement(n2.alternate));
      }).with({ type: ast.NodeType.LogicalExpression }, (n2) => {
        if (!("left" in n2)) return effect.Option.none();
        return effect.Option.orElse(checkIteratorElement(n2.left), () => checkIteratorElement(n2.right));
      }).otherwise(effect.Option.none);
    }
    function checkBlockStatement(node) {
      return ast.getNestedReturnStatements(node).reduce((acc, statement) => {
        if (!statement.argument) return acc;
        const maybeDescriptor = checkIteratorElement(statement.argument);
        if (effect.Option.isNone(maybeDescriptor)) return acc;
        const descriptor = maybeDescriptor.value;
        return [...acc, descriptor];
      }, []);
    }
    return {
      [`${childrenToArraySelector}:exit`]() {
        effect.MutableRef.set(isWithinChildrenToArrayRef, false);
      },
      ArrayExpression(node) {
        if (effect.MutableRef.get(isWithinChildrenToArrayRef)) return;
        const elements = node.elements.filter(ast.is(ast.NodeType.JSXElement));
        if (elements.length === 0) return;
        const initialScope = context.sourceCode.getScope(node);
        for (const element of elements) {
          if (!jsx.hasProp(element.openingElement.attributes, "key", context, initialScope)) {
            context.report({
              messageId: "NO_MISSING_KEY",
              node: element
            });
          }
        }
      },
      CallExpression(node) {
        const isMapCall = a({
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "map"
            }
          }
        }, node);
        const isArrayFromCall = a({
          type: ast.NodeType.CallExpression,
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "from"
            }
          }
        }, node);
        if (!isMapCall && !isArrayFromCall) return;
        if (effect.MutableRef.get(isWithinChildrenToArrayRef)) return;
        const fn = node.arguments[isMapCall ? 0 : 1];
        if (!ast.isOneOf([ast.NodeType.ArrowFunctionExpression, ast.NodeType.FunctionExpression])(fn)) return;
        if (fn.body.type === ast.NodeType.BlockStatement) {
          for (const descriptor of checkBlockStatement(fn.body)) {
            context.report(descriptor);
          }
          return;
        }
        effect.Option.map(checkExpression(fn.body), context.report);
      },
      JSXFragment(node) {
        if (effect.MutableRef.get(isWithinChildrenToArrayRef)) return;
        if (node.parent.type === ast.NodeType.ArrayExpression) {
          context.report({
            messageId: "NO_MISSING_KEY_WITH_FRAGMENT",
            node
          });
        }
      },
      [childrenToArraySelector]() {
        effect.MutableRef.set(isWithinChildrenToArrayRef, true);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME26 = "no-nested-components";
var no_nested_components_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of unstable nested components",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NESTED_COMPONENT: "Do not nest components inside other components. Move it to the top level.",
      NESTED_COMPONENT_IN_PROPS: "Do not nest components inside props. Move it to the top level or pass it as a prop."
    },
    schema: []
  },
  name: RULE_NAME26,
  create(context) {
    const hint = core.ERComponentHint.SkipMapCallback | core.ERComponentHint.SkipNullLiteral | core.ERComponentHint.SkipUndefinedLiteral | core.ERComponentHint.SkipBooleanLiteral | core.ERComponentHint.SkipStringLiteral | core.ERComponentHint.SkipNumberLiteral | core.ERComponentHint.StrictLogical | core.ERComponentHint.StrictConditional;
    const collector = core.useComponentCollector(context, hint);
    const collectorLegacy = core.useComponentCollectorLegacy();
    return {
      ...collector.listeners,
      ...collectorLegacy.listeners,
      "Program:exit"(node) {
        const functionComponents = Array.from(collector.ctx.getAllComponents(node).values());
        const classComponents = Array.from(collectorLegacy.ctx.getAllComponents(node).values());
        const isFunctionComponent = (node2) => {
          return ast.isFunction(node2) && functionComponents.some((component) => component.node === node2);
        };
        const isClassComponent8 = (node2) => {
          return ast.isClass(node2) && classComponents.some((component) => component.node === node2);
        };
        for (const { name: componentName, node: component } of functionComponents) {
          if (core.isDirectValueOfRenderPropertyLoose(component)) continue;
          if (effect.Option.isNone(componentName)) continue;
          const name2 = componentName.value;
          const isInsideProperty = component.parent.type === ast.NodeType.Property;
          const isInsideJSXPropValue = jsx.isInsidePropValue(component);
          if (isInsideJSXPropValue) {
            if (!core.isDeclaredInRenderPropLoose(component)) {
              context.report({
                data: {
                  name: name2
                },
                messageId: "NESTED_COMPONENT_IN_PROPS",
                node: component
              });
            }
            continue;
          }
          if (core.isInsideCreateElementProps(component, context)) {
            context.report({
              data: {
                name: name2
              },
              messageId: "NESTED_COMPONENT_IN_PROPS",
              node: component
            });
            continue;
          }
          const maybeParentComponent = ast.traverseUpGuard(component, isFunctionComponent);
          if (effect.Option.isSome(maybeParentComponent) && !core.isDirectValueOfRenderPropertyLoose(maybeParentComponent.value)) {
            context.report({
              data: {
                name: name2
              },
              messageId: isInsideProperty ? "NESTED_COMPONENT_IN_PROPS" : "NESTED_COMPONENT",
              node: component
            });
            continue;
          }
          const isInsideClassComponentRenderMethod = core.isInsideRenderMethod(component);
          if (isInsideClassComponentRenderMethod) {
            context.report({
              data: {
                name: name2
              },
              messageId: "NESTED_COMPONENT",
              node: component
            });
          }
        }
        for (const { name: name2, node: component } of classComponents) {
          if (effect.Option.isNone(ast.traverseUp(component, (node2) => isClassComponent8(node2) || isFunctionComponent(node2)))) continue;
          context.report({
            data: {
              name: effect.Option.getOrElse(() => "unknown")(name2)
            },
            messageId: "NESTED_COMPONENT",
            node: component
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME27 = "no-redundant-should-component-update";
function isShouldComponentUpdate(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "shouldComponentUpdate";
}
var no_redundant_should_component_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'shouldComponentUpdate' in class component extends 'React.PureComponent'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_REDUNDANT_SHOULD_COMPONENT_UPDATE: "'{{componentName}}' does not need 'shouldComponentUpdate' when extending 'React.PureComponent'."
    },
    schema: []
  },
  name: RULE_NAME27,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { name: name2, flag, node: component } of components.values()) {
          if (!(flag & core.ERClassComponentFlag.PureComponent)) continue;
          const { body } = component.body;
          for (const member of body) {
            if (isShouldComponentUpdate(member)) {
              context.report({
                data: {
                  componentName: effect.Option.getOrElse(() => "PureComponent")(name2)
                },
                messageId: "NO_REDUNDANT_SHOULD_COMPONENT_UPDATE",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME28 = "no-set-state-in-component-did-mount";
function isThisSetState2(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && callee.object.type === ast.NodeType.ThisExpression && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function isComponentDidMount(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentDidMount";
}
var no_set_state_in_component_did_mount_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'setState' in 'componentDidMount'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_SET_STATE_IN_COMPONENT_DID_MOUNT: "Do not call `this.setState` in `componentDidMount` outside of functions, such as callbacks."
    },
    schema: []
  },
  name: RULE_NAME28,
  create(context) {
    return {
      CallExpression(node) {
        if (!isThisSetState2(node)) return;
        const maybeParentClass = ast.traverseUp(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (effect.Option.isNone(maybeParentClass)) return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass)) return;
        const maybeParentMethod = ast.traverseUp(node, isComponentDidMount);
        if (effect.Option.isNone(maybeParentMethod)) return;
        const parentMethod = maybeParentMethod.value;
        if (parentMethod.parent !== parentClass.body) return;
        if (context.sourceCode.getScope(node).upper !== context.sourceCode.getScope(parentMethod)) return;
        context.report({
          messageId: "NO_SET_STATE_IN_COMPONENT_DID_MOUNT",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME29 = "no-set-state-in-component-did-update";
function isThisSetState3(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && callee.object.type === ast.NodeType.ThisExpression && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function isComponentDidUpdate(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentDidUpdate";
}
var no_set_state_in_component_did_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'setState' in 'componentDidUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_SET_STATE_IN_COMPONENT_DID_UPDATE: "Do not call `this.setState` in `componentDidUpdate` outside of functions, such as callbacks."
    },
    schema: []
  },
  name: RULE_NAME29,
  create(context) {
    return {
      CallExpression(node) {
        if (!isThisSetState3(node)) return;
        const maybeParentClass = ast.traverseUp(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (effect.Option.isNone(maybeParentClass)) return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass)) return;
        const maybeParentMethod = ast.traverseUp(node, isComponentDidUpdate);
        if (effect.Option.isNone(maybeParentMethod)) return;
        const parentMethod = maybeParentMethod.value;
        if (parentMethod.parent !== parentClass.body) return;
        if (context.sourceCode.getScope(node).upper !== context.sourceCode.getScope(parentMethod)) return;
        context.report({
          messageId: "NO_SET_STATE_IN_COMPONENT_DID_UPDATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME30 = "no-set-state-in-component-will-update";
function isThisSetState4(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && callee.object.type === ast.NodeType.ThisExpression && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function isComponentWillUpdate3(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillUpdate";
}
var no_set_state_in_component_will_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'setState' in 'componentWillUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_SET_STATE_IN_COMPONENT_WILL_UPDATE: "Do not call `this.setState` in `componentWillUpdate` outside of functions, such as callbacks."
    },
    schema: []
  },
  name: RULE_NAME30,
  create(context) {
    return {
      CallExpression(node) {
        if (!isThisSetState4(node)) return;
        const maybeParentClass = ast.traverseUp(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (effect.Option.isNone(maybeParentClass)) return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass)) return;
        const maybeParentMethod = ast.traverseUp(node, isComponentWillUpdate3);
        if (effect.Option.isNone(maybeParentMethod)) return;
        const parentMethod = maybeParentMethod.value;
        if (parentMethod.parent !== parentClass.body) return;
        if (context.sourceCode.getScope(node).upper !== context.sourceCode.getScope(parentMethod)) return;
        context.report({
          messageId: "NO_SET_STATE_IN_COMPONENT_WILL_UPDATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME31 = "no-string-refs";
function containsStringLiteral({ value }) {
  return value?.type === ast.NodeType.Literal && effect.Predicate.isString(value.value);
}
function containsStringExpressionContainer({ value }) {
  if (value?.type !== ast.NodeType.JSXExpressionContainer) return false;
  if (value.expression.type === ast.NodeType.Literal) return effect.Predicate.isString(value.expression.value);
  return value.expression.type === ast.NodeType.TemplateLiteral;
}
var no_string_refs_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow using deprecated string refs",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_STRING_REFS: "[Deprecated] Use callback refs instead."
    },
    schema: []
  },
  name: RULE_NAME31,
  create(context) {
    return {
      JSXAttribute(node) {
        if (node.name.name !== "ref") return;
        if (containsStringLiteral(node) || containsStringExpressionContainer(node)) {
          context.report({
            messageId: "NO_STRING_REFS",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME32 = "no-unsafe-component-will-mount";
function isUnsafeComponentWillMount(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "UNSAFE_componentWillMount";
}
var no_unsafe_component_will_mount_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'UNSAFE_componentWillMount'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSAFE_COMPONENT_WILL_MOUNT: "Do not use 'UNSAFE_componentWillMount'."
    },
    schema: []
  },
  name: RULE_NAME32,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isUnsafeComponentWillMount(member)) {
              context.report({
                messageId: "NO_UNSAFE_COMPONENT_WILL_MOUNT",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME33 = "no-unsafe-component-will-receive-props";
function isUnsafeComponentWillReceiveProps(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "UNSAFE_componentWillReceiveProps";
}
var no_unsafe_component_will_receive_props_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'UNSAFE_componentWillReceiveProps'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSAFE_COMPONENT_WILL_RECEIVE_PROPS: "Do not use 'UNSAFE_componentWillReceiveProps'."
    },
    schema: []
  },
  name: RULE_NAME33,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isUnsafeComponentWillReceiveProps(member)) {
              context.report({
                messageId: "NO_UNSAFE_COMPONENT_WILL_RECEIVE_PROPS",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME34 = "no-unsafe-component-will-update";
function isUnsafeComponentWillMount2(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "UNSAFE_componentWillUpdate";
}
var no_unsafe_component_will_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'UNSAFE_componentWillUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSAFE_COMPONENT_WILL_UPDATE: "Do not use 'UNSAFE_componentWillUpdate'."
    },
    schema: []
  },
  name: RULE_NAME34,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isUnsafeComponentWillMount2(member)) {
              context.report({
                messageId: "NO_UNSAFE_COMPONENT_WILL_UPDATE",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME35 = "no-unstable-context-value";
var no_unstable_context_value_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow passing constructed values to context providers",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSTABLE_CONTEXT_VALUE: "A/an '{{type}}' passed as the value prop to the context provider should not be constructed. It will change on every render.",
      NO_UNSTABLE_CONTEXT_VALUE_WITH_FUNCTION: "A/an '{{type}}' passed as the value prop to the context provider should not be constructed. It will change on every render. Consider wrapping it in a useCallback hook.",
      NO_UNSTABLE_CONTEXT_VALUE_WITH_IDENTIFIER: "A/an '{{type}}' passed as the value prop to the context provider should not be constructed. It will change on every render. Consider wrapping it in a useMemo hook."
    },
    schema: []
  },
  name: RULE_NAME35,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const possibleValueConstructions = /* @__PURE__ */ new Map();
    return {
      ...listeners,
      JSXOpeningElement(node) {
        const openingElementName = node.name;
        if (openingElementName.type !== ast.NodeType.JSXMemberExpression) return;
        if (openingElementName.property.name !== "Provider") return;
        const maybeJSXValueAttribute = effect.Option.fromNullable(
          node.attributes.find((attribute) => {
            return attribute.type === ast.NodeType.JSXAttribute && attribute.name.name === "value";
          })
        );
        if (effect.Option.isNone(maybeJSXValueAttribute) || !("value" in maybeJSXValueAttribute.value)) return;
        const valueNode = maybeJSXValueAttribute.value.value;
        if (valueNode?.type !== ast.NodeType.JSXExpressionContainer) return;
        const valueExpression = valueNode.expression;
        const construction = ast.inspectConstruction(valueExpression, context);
        if (construction._tag === "None") return;
        effect.Option.map(
          ctx.getCurrentFunction(),
          ([currentFn]) => possibleValueConstructions.set(currentFn, [
            ...possibleValueConstructions.get(currentFn) ?? [],
            construction
          ])
        );
      },
      "Program:exit"(node) {
        const components = Array.from(ctx.getAllComponents(node).values());
        for (const { node: component } of components) {
          const constructions = possibleValueConstructions.get(component);
          if (!constructions) continue;
          for (const construction of constructions) {
            if (construction._tag === "None") continue;
            const { _tag, node: constructionNode } = construction;
            const messageId = _tag.startsWith("Function") ? "NO_UNSTABLE_CONTEXT_VALUE_WITH_FUNCTION" : "NO_UNSTABLE_CONTEXT_VALUE_WITH_IDENTIFIER";
            context.report({
              data: {
                type: ast.readableNodeType(constructionNode)
              },
              messageId,
              node: constructionNode
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME36 = "no-unstable-default-props";
var no_unstable_default_props_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of unstable value as default param in function component",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSTABLE_DEFAULT_PROPS: "A/an '{{forbiddenType}}' as default prop. This could lead to potential infinite render loop in React. Use a variable instead of '{{forbiddenType}}'."
    },
    schema: []
  },
  name: RULE_NAME36,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const possibleDestructuringDeclarators = /* @__PURE__ */ new WeakMap();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { params } = component;
          const [props] = params;
          if (!props) continue;
          const properties = $(props).with({ type: ast.NodeType.ObjectPattern }, ({ properties: properties2 }) => properties2).with({ type: ast.NodeType.Identifier }, ({ name: name2 }) => {
            const variableDeclarators = possibleDestructuringDeclarators.get(component);
            if (!variableDeclarators) return [];
            const declarators = variableDeclarators.filter((d2) => d2.init.name === name2);
            return declarators.flatMap((d2) => d2.id.properties);
          }).otherwise(() => []);
          for (const prop of properties) {
            if (prop.type !== ast.NodeType.Property || prop.value.type !== ast.NodeType.AssignmentPattern) continue;
            const { value } = prop;
            const { right } = value;
            const construction = ast.inspectConstruction(value, context, ast.ConstructionHint.StrictCallExpression);
            if (construction._tag === "None") continue;
            const forbiddenType = ast.readableNodeType(right);
            context.report({
              data: {
                forbiddenType
              },
              messageId: "NO_UNSTABLE_DEFAULT_PROPS",
              node: right
            });
          }
        }
      },
      "VariableDeclarator[id.type='ObjectPattern'][init.type='Identifier']"(node) {
        effect.Option.map(
          ctx.getCurrentFunction(),
          ([currentFn]) => possibleDestructuringDeclarators.set(currentFn, [
            ...possibleDestructuringDeclarators.get(currentFn) ?? [],
            node
          ])
        );
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME37 = "no-unused-class-component-members";
var LIFECYCLE_METHODS = /* @__PURE__ */ new Set([
  "constructor",
  "componentDidCatch",
  "componentDidMount",
  "componentDidUpdate",
  "componentWillMount",
  "componentWillReceiveProps",
  "componentWillUnmount",
  "componentWillUpdate",
  "getSnapshotBeforeUpdate",
  "shouldComponentUpdate",
  "UNSAFE_componentWillMount",
  "UNSAFE_componentWillReceiveProps",
  "UNSAFE_componentWillUpdate",
  "state",
  "render"
]);
function isKeyLiteralLike2(node, property) {
  return property.type === ast.NodeType.Literal || property.type === ast.NodeType.TemplateLiteral && property.expressions.length === 0 || !node.computed && property.type === ast.NodeType.Identifier;
}
function getName3(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName3(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return effect.Option.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return effect.Option.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return effect.Option.fromNullable(node.quasis[0]?.value.raw);
  }
  return effect.Option.none();
}
var no_unused_class_component_members_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow unused class component members",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNUSED_CLASS_COMPONENT_MEMBERS: "Unused method or property '{{methodName}}'' of class '{{className}}'."
    },
    schema: []
  },
  name: RULE_NAME37,
  create(context) {
    const classStack = effect.MutableList.make();
    const methodStack = effect.MutableList.make();
    const propertyDefs = /* @__PURE__ */ new WeakMap();
    const propertyUsages = /* @__PURE__ */ new WeakMap();
    function classEnter(node) {
      effect.MutableList.append(classStack, node);
      if (!core.isClassComponent(node)) return;
      propertyDefs.set(node, /* @__PURE__ */ new Set());
      propertyUsages.set(node, /* @__PURE__ */ new Set());
    }
    function classExit() {
      const currentClass = effect.MutableList.pop(classStack);
      if (!currentClass || !core.isClassComponent(currentClass)) return;
      const className = effect.Option.map(ast.getClassIdentifier(currentClass), (id) => id.name);
      const defs = propertyDefs.get(currentClass);
      const usages = propertyUsages.get(currentClass);
      if (!defs) return;
      for (const def of defs) {
        const name2 = getName3(def);
        if (effect.Option.isNone(name2)) continue;
        if (!!usages?.has(name2.value) || LIFECYCLE_METHODS.has(name2.value)) continue;
        context.report({
          data: {
            className: effect.Option.getOrElse(className, () => "Component"),
            methodName: name2.value
          },
          messageId: "NO_UNUSED_CLASS_COMPONENT_MEMBERS",
          node: def
        });
      }
    }
    function methodEnter(node) {
      effect.MutableList.append(methodStack, node);
      const currentClass = effect.MutableList.tail(classStack);
      if (!currentClass || !core.isClassComponent(currentClass)) return;
      if (node.static) return;
      if (isKeyLiteralLike2(node, node.key)) {
        propertyDefs.get(currentClass)?.add(node.key);
      }
    }
    function methodExit() {
      effect.MutableList.pop(methodStack);
    }
    return {
      ClassDeclaration: classEnter,
      "ClassDeclaration:exit": classExit,
      ClassExpression: classEnter,
      "ClassExpression:exit": classExit,
      MemberExpression(node) {
        const currentClass = effect.MutableList.tail(classStack);
        const currentMethod = effect.MutableList.tail(methodStack);
        if (!currentClass || !currentMethod) return;
        if (!core.isClassComponent(currentClass) || currentMethod.static) return;
        if (!ast.isThisExpression(node.object) || !isKeyLiteralLike2(node, node.property)) return;
        if (node.parent.type === ast.NodeType.AssignmentExpression && node.parent.left === node) {
          propertyDefs.get(currentClass)?.add(node.property);
          return;
        }
        effect.Option.map(getName3(node.property), (name2) => propertyUsages.get(currentClass)?.add(name2));
      },
      MethodDefinition: methodEnter,
      "MethodDefinition:exit": methodExit,
      PropertyDefinition: methodEnter,
      "PropertyDefinition:exit": methodExit,
      VariableDeclarator(node) {
        const currentClass = effect.MutableList.tail(classStack);
        const currentMethod = effect.MutableList.tail(methodStack);
        if (!currentClass || !currentMethod) return;
        if (!core.isClassComponent(currentClass) || currentMethod.static) return;
        if (node.init && ast.isThisExpression(node.init) && node.id.type === ast.NodeType.ObjectPattern) {
          for (const prop of node.id.properties) {
            if (prop.type === ast.NodeType.Property && isKeyLiteralLike2(prop, prop.key)) {
              effect.Option.map(getName3(prop.key), (name2) => propertyUsages.get(currentClass)?.add(name2));
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME38 = "no-unused-state";
function isKeyLiteralLike3(node, property) {
  return property.type === ast.NodeType.Literal || property.type === ast.NodeType.TemplateLiteral && property.expressions.length === 0 || !node.computed && property.type === ast.NodeType.Identifier;
}
function getName4(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName4(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return effect.Option.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return effect.Option.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return effect.Option.fromNullable(node.quasis[0]?.value.raw);
  }
  return effect.Option.none();
}
function isAssignmentToThisState2(node) {
  const { left } = node;
  return left.type === ast.NodeType.MemberExpression && ast.isThisExpression(left.object) && effect.Option.exists(getName4(left.property), (name2) => name2 === "state");
}
var isGetDerivedStateFromProps = a({
  key: {
    name: "getDerivedStateFromProps"
  },
  static: true,
  value: {
    params: N.array()
  }
});
var no_unused_state_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "Prevents unused state of class component.",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNUSED_STATE: "Unused class component state."
    },
    schema: []
  },
  name: RULE_NAME38,
  create(context) {
    const classStack = effect.MutableList.make();
    const methodStack = effect.MutableList.make();
    const constructorStack = effect.MutableList.make();
    const stateDefs = /* @__PURE__ */ new WeakMap();
    function classEnter(node) {
      effect.MutableList.append(classStack, node);
    }
    function classExit() {
      const currentClass = effect.MutableList.pop(classStack);
      if (!currentClass || !core.isClassComponent(currentClass)) return;
      const className = effect.Option.map(ast.getClassIdentifier(currentClass), (id) => id.name);
      const [def, isUsed] = stateDefs.get(currentClass) ?? [effect.Option.none(), false];
      if (effect.Option.isNone(def) || isUsed) return;
      context.report({
        data: {
          className: effect.Option.getOrElse(className, () => "Component")
        },
        messageId: "NO_UNUSED_STATE",
        node: def.value
      });
    }
    function methodEnter(node) {
      effect.MutableList.append(methodStack, node);
      const currentClass = effect.MutableList.tail(classStack);
      if (!currentClass || !core.isClassComponent(currentClass)) return;
      if (node.static) {
        if (isGetDerivedStateFromProps(node) && node.value.params.length > 1) {
          const [def] = stateDefs.get(currentClass) ?? [effect.Option.none()];
          stateDefs.set(currentClass, [def, true]);
        }
        return;
      }
      if (effect.Option.exists(getName4(node.key), (name2) => name2 === "state")) {
        stateDefs.set(currentClass, [effect.Option.some(node.key), false]);
      }
    }
    function methodExit() {
      effect.MutableList.pop(methodStack);
    }
    function constructorEnter(node) {
      effect.MutableList.append(constructorStack, node);
    }
    function constructorExit() {
      effect.MutableList.pop(constructorStack);
    }
    return {
      AssignmentExpression(node) {
        if (!isAssignmentToThisState2(node)) return;
        const currentClass = effect.MutableList.tail(classStack);
        if (!currentClass || !core.isClassComponent(currentClass)) return;
        const currentConstructor = effect.MutableList.tail(constructorStack);
        if (!currentConstructor || !currentClass.body.body.includes(currentConstructor)) return;
        const [_2, isUsed] = stateDefs.get(currentClass) ?? [effect.Option.none(), false];
        stateDefs.set(currentClass, [effect.Option.some(node.left), isUsed]);
      },
      ClassDeclaration: classEnter,
      "ClassDeclaration:exit": classExit,
      ClassExpression: classEnter,
      "ClassExpression:exit": classExit,
      MemberExpression(node) {
        if (!ast.isThisExpression(node.object)) return;
        if (!effect.Option.exists(getName4(node.property), (name2) => name2 === "state")) return;
        const currentClass = effect.MutableList.tail(classStack);
        if (!currentClass || !core.isClassComponent(currentClass)) return;
        const currentMethod = effect.MutableList.tail(methodStack);
        if (!currentMethod || currentMethod.static) return;
        if (currentMethod === effect.MutableList.tail(constructorStack)) return;
        if (!currentClass.body.body.includes(currentMethod)) return;
        const [def] = stateDefs.get(currentClass) ?? [effect.Option.none(), false];
        stateDefs.set(currentClass, [def, true]);
      },
      MethodDefinition: methodEnter,
      "MethodDefinition:exit": methodExit,
      "MethodDefinition[key.name='constructor']": constructorEnter,
      "MethodDefinition[key.name='constructor']:exit": constructorExit,
      PropertyDefinition: methodEnter,
      "PropertyDefinition:exit": methodExit,
      VariableDeclarator(node) {
        const currentClass = effect.MutableList.tail(classStack);
        if (!currentClass || !core.isClassComponent(currentClass)) return;
        const currentMethod = effect.MutableList.tail(methodStack);
        if (!currentMethod || currentMethod.static) return;
        if (currentMethod === effect.MutableList.tail(constructorStack)) return;
        if (!currentClass.body.body.includes(currentMethod)) return;
        if (!(node.init && ast.isThisExpression(node.init) && node.id.type === ast.NodeType.ObjectPattern)) return;
        const hasState = node.id.properties.some((prop) => {
          if (prop.type === ast.NodeType.Property && isKeyLiteralLike3(prop, prop.key)) {
            return effect.Option.exists(getName4(prop.key), (name2) => name2 === "state");
          }
          return false;
        });
        if (!hasState) return;
        const [def] = stateDefs.get(currentClass) ?? [effect.Option.none(), false];
        stateDefs.set(currentClass, [def, true]);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME39 = "no-useless-fragment";
function isFragmentWithOnlyTextAndIsNotChild(node) {
  return node.children.length === 1 && jsx.isLiteral(node.children[0]) && !(node.parent.type === ast.NodeType.JSXElement || node.parent.type === ast.NodeType.JSXFragment);
}
function containsCallExpression(node) {
  return node.type === ast.NodeType.JSXExpressionContainer && node.expression.type === ast.NodeType.CallExpression;
}
function isFragmentHasLessThanTwoChildren(node) {
  const nonPaddingChildren = node.children.filter(
    (child) => !jsx.isPaddingSpaces(child)
  );
  if (nonPaddingChildren.length === 1 && nonPaddingChildren[0]) {
    return !containsCallExpression(nonPaddingChildren[0]);
  }
  return nonPaddingChildren.length === 0;
}
function isFragmentWithSingleExpression(node) {
  const children = node.children.filter((child) => !jsx.isPaddingSpaces(child));
  return children.length === 1 && children[0]?.type === ast.NodeType.JSXExpressionContainer;
}
var no_useless_fragment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow unnecessary fragments",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_USELESS_FRAGMENT: "A fragment containing a single element is usually unnecessary.",
      NO_USELESS_FRAGMENT_IN_BUILT_IN: "Passing a fragment to a host component is unnecessary."
    },
    schema: []
  },
  name: RULE_NAME39,
  create(context) {
    function checkNode(node) {
      const initialScope = context.sourceCode.getScope(node);
      if (node.type === ast.NodeType.JSXElement && jsx.hasProp(
        node.openingElement.attributes,
        "key",
        context,
        initialScope
      )) {
        return;
      }
      if (isFragmentHasLessThanTwoChildren(node) && !isFragmentWithOnlyTextAndIsNotChild(node) && !(isFragmentWithSingleExpression(node))) {
        context.report({
          messageId: "NO_USELESS_FRAGMENT",
          node
        });
      }
      if (node.parent.type === ast.NodeType.JSXElement && jsx.isJSXElementOfBuiltinComponent(node.parent)) {
        context.report({
          messageId: "NO_USELESS_FRAGMENT_IN_BUILT_IN",
          node
        });
      }
    }
    return {
      JSXElement(node) {
        if (core.isFragmentElement(node, context)) {
          checkNode(node);
        }
      },
      JSXFragment: checkNode
    };
  },
  defaultOptions: []
});
var RULE_NAME40 = "prefer-destructuring-assignment";
function isMemberExpressionWithObjectName(node) {
  return node.object.type === ast.NodeType.Identifier && "name" in node.object;
}
var prefer_destructuring_assignment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce using destructuring assignment in component props and context",
      recommended: "recommended"
    },
    messages: {
      PREFER_DESTRUCTURING_ASSIGNMENT: "Prefer destructuring assignment."
    },
    schema: []
  },
  name: RULE_NAME40,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const memberExpressionWithNames = [];
    return {
      ...listeners,
      MemberExpression(node) {
        if (isMemberExpressionWithObjectName(node)) {
          const scope = context.sourceCode.getScope(node);
          memberExpressionWithNames.push([scope, node]);
        }
      },
      "Program:exit"(node) {
        const components = Array.from(ctx.getAllComponents(node).values());
        function isFunctionComponent(block) {
          if (!ast.isFunction(block)) return false;
          const maybeId = ast.getFunctionIdentifier(block);
          return effect.Option.isSome(maybeId) && core.isComponentName(maybeId.value.name) && components.some((component) => component.node === block);
        }
        for (const [scope, memberExpression] of memberExpressionWithNames) {
          const scopeRef = effect.MutableRef.make(scope);
          const isComponentRef = effect.MutableRef.make(isFunctionComponent(scope.block));
          while (!effect.MutableRef.get(isComponentRef) && effect.MutableRef.get(scopeRef).upper && effect.MutableRef.get(scopeRef).upper !== effect.MutableRef.get(scopeRef)) {
            effect.MutableRef.set(scopeRef, effect.MutableRef.get(scopeRef).upper);
            effect.MutableRef.set(isComponentRef, isFunctionComponent(effect.MutableRef.get(scopeRef).block));
          }
          if (!effect.MutableRef.get(isComponentRef)) continue;
          const component = effect.MutableRef.get(scopeRef).block;
          if (!("params" in component)) continue;
          const [props, ctx2] = component.params;
          const isMatch = a({ name: memberExpression.object.name });
          if (isMatch(props)) {
            context.report({
              messageId: "PREFER_DESTRUCTURING_ASSIGNMENT",
              node: memberExpression
            });
          }
          if (isMatch(ctx2)) {
            context.report({
              messageId: "PREFER_DESTRUCTURING_ASSIGNMENT",
              node: memberExpression
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME41 = "prefer-shorthand-boolean";
var prefer_shorthand_boolean_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce boolean attributes notation in JSX",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      PREFER_SHORTHAND_BOOLEAN: "Prefer using shorthand boolean attribute '{{propName}}'"
    },
    schema: []
  },
  name: RULE_NAME41,
  create(context) {
    return {
      JSXAttribute(node) {
        const { value } = node;
        const propName = jsx.getPropName(node);
        const hasValueTrue = value?.type === ast.NodeType.JSXExpressionContainer && value.expression.type === ast.NodeType.Literal && value.expression.value === true;
        if (!hasValueTrue) return;
        context.report({
          data: {
            propName
          },
          messageId: "PREFER_SHORTHAND_BOOLEAN",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME42 = "prefer-shorthand-fragment";
var prefer_shorthand_fragment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce using fragment syntax instead of Fragment component",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      PREFER_SHORTHAND_FRAGMENT: "Prefer shorthand fragment syntax instead of fragment component."
    },
    schema: []
  },
  name: RULE_NAME42,
  create(context) {
    return {
      JSXElement(node) {
        if (core.isFragmentElement(node, context)) {
          const hasAttributes = node.openingElement.attributes.length > 0;
          if (hasAttributes) return;
          context.report({
            messageId: "PREFER_SHORTHAND_FRAGMENT",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});

// src/index.ts
var meta = {
  name,
  version
};
var rules = {
  "avoid-shorthand-boolean": avoid_shorthand_boolean_default,
  "avoid-shorthand-fragment": avoid_shorthand_fragment_default,
  "ensure-forward-ref-using-ref": ensure_forward_ref_using_ref_default,
  "no-access-state-in-setstate": no_access_state_in_setstate_default,
  "no-array-index-key": no_array_index_key_default,
  "no-children-count": no_children_count_default,
  "no-children-for-each": no_children_for_each_default,
  "no-children-map": no_children_map_default,
  "no-children-only": no_children_only_default,
  "no-children-prop": no_children_prop_default,
  "no-children-to-array": no_children_to_array_default,
  "no-class-component": no_class_component_default,
  "no-clone-element": no_clone_element_default,
  "no-comment-textnodes": no_comment_textnodes_default,
  "no-complicated-conditional-rendering": no_complicated_conditional_rendering_default,
  "no-component-will-mount": no_component_will_mount_default,
  "no-component-will-receive-props": no_component_will_receive_props_default,
  "no-component-will-update": no_component_will_update_default,
  "no-create-ref": no_create_ref_default,
  "no-direct-mutation-state": no_direct_mutation_state_default,
  "no-duplicate-key": no_duplicate_key_default,
  "no-implicit-key": no_implicit_key_default,
  "no-leaked-conditional-rendering": no_leaked_conditional_rendering_default,
  "no-missing-component-display-name": no_missing_component_display_name_default,
  "no-missing-key": no_missing_key_default,
  "no-nested-components": no_nested_components_default,
  "no-redundant-should-component-update": no_redundant_should_component_update_default,
  "no-set-state-in-component-did-mount": no_set_state_in_component_did_mount_default,
  "no-set-state-in-component-did-update": no_set_state_in_component_did_update_default,
  "no-set-state-in-component-will-update": no_set_state_in_component_will_update_default,
  "no-string-refs": no_string_refs_default,
  "no-unsafe-component-will-mount": no_unsafe_component_will_mount_default,
  "no-unsafe-component-will-receive-props": no_unsafe_component_will_receive_props_default,
  "no-unsafe-component-will-update": no_unsafe_component_will_update_default,
  "no-unstable-context-value": no_unstable_context_value_default,
  "no-unstable-default-props": no_unstable_default_props_default,
  "no-unused-class-component-members": no_unused_class_component_members_default,
  "no-unused-state": no_unused_state_default,
  "no-useless-fragment": no_useless_fragment_default,
  "prefer-destructuring-assignment": prefer_destructuring_assignment_default,
  "prefer-shorthand-boolean": prefer_shorthand_boolean_default,
  "prefer-shorthand-fragment": prefer_shorthand_fragment_default
};

exports.meta = meta;
exports.rules = rules;
